
#pragma once

#ifndef VEC2_HPP
#define VEC2_HPP

#include <ostream>

/** vec2 class model a 2D vector
    Vector function are provided as standard functions. */
class vec2
{
    /** Internal x data*/
    float x_data;
    /** Internal y data*/
    float y_data;

public:
    /** Constructor to (0,0) vector*/
    vec2();
    /** Constructor to (x,y) vector*/
    vec2(float x_param,float y_param);

    /** get x value */
    float x() const;
    /** get y value */
    float y() const;
    /** set x value */
    float& x();
    /** set t value */
    float& y();

    /** get i_th value */
    float operator[](int k) const;
    /** set i_th value */
    float& operator[](int k);

    /** get i_th value */
    float operator()(int k) const;
    /** set i_th value */
    float& operator()(int k);
};

/** Add v2 to v1*/
vec2& operator+=(vec2& v1,vec2 const& v2);
/** Add two vectors*/
vec2 operator+(vec2 const& v1,vec2 const& v2);

/** Unary negation */
vec2 operator-(vec2 const& v);

/** Substract v2 to v1*/
vec2& operator-=(vec2& v1,vec2 const& v2);
/** Substract two vectors*/
vec2 operator-(vec2 const& v1,vec2 const& v2);

/** Multiply scalar s to the vector */
vec2& operator*=(vec2& v,float s);
/** Return multiplication of scalar s to the vector */
vec2 operator*(vec2 const& v,float s);
/** Return multiplication of scalar s to the vector */
vec2 operator*(float s,vec2 const& v);

/** Divide vector v by the scalar s */
vec2& operator/=(vec2& v,float s);
/** Return vector v divided by the scalar s */
vec2 operator/(vec2 const& v,float s);

/** Check if two vectors are equal */
bool operator==(vec2 const& v1,vec2 const& v2);
/** Check if two vectors are not equal */
bool operator!=(vec2 const& v1,vec2 const& v2);

/** Export vector values to stream (to print for instance) */
std::ostream& operator<<(std::ostream& s,vec2 const& v);

/** Dot product between two vectors */
float dot(vec2 const& v1,vec2 const& v2);


#endif
