
#pragma once

#ifndef VEC2_HPP
#define VEC2_HPP

#include <ostream>

/** vec2 class model a 2D vector
    Vector function are provided as member functions. */
class vec2
{
    /** Internal x data*/
    float x_data;
    /** Internal y data*/
    float y_data;

public:
    /** Constructor to (0,0) vector*/
    vec2();
    /** Constructor to (x,y) vector*/
    vec2(float x_param,float y_param);

    /** get x value */
    float x() const;
    /** get y value */
    float y() const;
    /** set x value */
    float& x();
    /** set t value */
    float& y();

    /** get i_th value */
    float operator[](int k) const;
    /** set i_th value */
    float& operator[](int k);

    /** get i_th value */
    float operator()(int k) const;
    /** set i_th value */
    float& operator()(int k);




    /** Add v to current vector*/
    vec2& operator+=(vec2 const& v);
    /** Add two vectors*/
    vec2 operator+(vec2 const& v) const;

    /** Unary negation */
    vec2 operator-() const;

    /** Substract v to current vector*/
    vec2& operator-=(vec2 const& v);
    /** Substract two vectors*/
    vec2 operator-(vec2 const& v) const;

    /** Multiply scalar s to the current vector */
    vec2& operator*=(float s);
    /** Return multiplication of scalar s to the vector */
    vec2 operator*(float s) const;

    /** Divide current vector by the scalar s */
    vec2& operator/=(float s);
    /** Return vector divided by the scalar s */
    vec2 operator/(float s) const;

    /** Check if vector v equals the current vector */
    bool operator==(vec2 const& v) const;
    /** Check if vector v is not equal to the current vector */
    bool operator!=(vec2 const& v) const;

    /** Dot product between the current vector and v */
    float dot(vec2 const& v) const;
};

/** Return multiplication of scalar s to the vector
 * \note This cannot be done as a member function */
vec2 operator*(float s,vec2 const& v);

/** Export vector values to stream (to print for instance)
 * \note THis cannot be done as a member function */
std::ostream& operator<<(std::ostream& s,vec2 const& v);

#endif
